function sysm = sysmat(pf,ra,pia,gamq,neta,nsig)
% General format
%
% TRANSITION: x(t)
%   x(t) = h(x(t-1)) + R*vep(t)                                
%   z(t) = g(x(t));
%   vep(t) ~ iid N(0,I)
%   z:  [y pi R]' (delete 'c' from z for it doesn't affect observations)
%   x:  [y(-1) R(-1) e g z]'
%

%-----------------------------------------------
% delete consumption from x_c
% which not used in measurement
%-----------------------------------------------
pf.gx(1,:)    = [];
pf.gxx(1,:,:) = [];
pf.gss(1)     = [];

%-----------------------------------------------
% State variables
%-----------------------------------------------
sysm.hc = pf.hss/2*nsig^2;
sysm.hl = pf.hx;
sysm.hq = permute(pf.hxx(1:2,:,:),[2 3 1]);
sysm.hq = sysm.hq(:,:)'/2;
sysm.R = neta*nsig;
%-----------------------------------------------
% Control variables
%-----------------------------------------------
sysm.gc  = pf.gss/2*nsig^2;
sysm.gl = pf.gx;
sysm.gq  = permute(pf.gxx,[2 3 1]);
sysm.gq = sysm.gq(:,:)'/2;
%-----------------------------------------------
% Measurement equations
%
%  Ygr = gam + 100*Y - 100*Y(-1) + 100*z;
%  INF = pist + 400*pi;
%  R   = pist + rst + 4*gam + 400*R;
%
%  y(t) = sysm.c + sysm.zz*z(t) sysm.zx*x(t)+ sysm.H*u(t)
%  u(t) ~ N(0,I)
%-----------------------------------------------
%   y:  [dlog(gdp), inflation, interest rate]'
%   z:  [y pi R]' (delete 'c' from z for it doesn't affect observations)
%   x:  [y(-1) R(-1) e g z]'

sysm.c = [gamq;pia;pia+ra+4*gamq];
sysm.zz = [100,0,0;0,400,0;0,0,400];
sysm.zx = [-100,0,0,0,100;0,0,0,0,0;0,0,0,0,0];
sysm.H = sqrt(0.1*diag([0.2872,0.8090,4.7202]));  
end