function [y] = prior(x)
%  The priors considered are:
%  Input: nx times k matrix, contains k column vectors or k times nx matrix

% Hyperparameters
hp = load('para_prior.txt');
nx = size(hp,1);

if size(x,2)~= nx
    x = x';
end

k  = size(x,1);
y = zeros(k,1);

for i = 1:nx

	dist = hp(i,1);
	para1 = hp(i,2);
	para2 = hp(i,3);

    switch dist
        case 1 % BETA prior	
            a = (1-para1)*para1^2/para2^2 - para1;
		    b = a*(1/para1 - 1);
		    y = y + lnpdfbeta(x(:,i),a,b);
        
        case 2 % GAMMA prior
		    b = para2^2/para1;
		    a = para1/b;
		    y = y + lnpdfgam(x(:,i),a,b);
        
        case 3 % NORMAL prior		
            a = para1;
		    b = para2;
		    y = y + lnpdfnorm(x(:,i),a,b);
            
        case 4 	% INVGAMMA prior
            a = para1;
		    b = para2;
		    y = y + lnpdfig(x(:,i),a,b);
            
        case 5 % uniform
            a = para1;
		    b = para2;
		    y = y + log(b-a);
    end       
end
end

% beta
function y=lnpdfbeta(x,a,b)
index = find(0<x&x<1);
y = repmat(-Inf,length(x),1);
y(index) = gammaln(a+b) - gammaln(a) - gammaln(b) + (a-1)*log(x(index)) + (b-1)*log(1-x(index));
end

% gamma
function y=lnpdfgam(x,a,b)
index = find(x>0);
y = repmat(-Inf,length(x),1);
y(index) = -gammaln(a) -a*log(b)+ (a-1)*log(x(index)) -x(index)/b ;
end

% inverse gamma
function y = lnpdfig(x,a,b)
index = find(x>0);
y = repmat(-Inf,length(x),1);
y(index) = log(2) - gammaln(b/2) + (b/2)*log(b*a^2/2) - ((b+1)/2)*log(x(index).^2) - b*a^2./(2*x(index).^2);
end

% normal
function y = lnpdfnorm(x,a,b)
y = -0.5*log(2*pi) - log(b) - 0.5*(x-a).^2/b^2;
end
